@@ -1,6 +1,10 @@
 from functools import partial
 from django.urls.resolvers import RoutePattern, RegexPattern, URLPattern, URLResolver
+import logging
 
+role_session_key = 'roles'
+role_app_key = 'permitted_roles'
+role_url_key = 'roles'
 
 class JvURLPattern(URLPattern):
     def __init__(self, pattern, callback, default_args=None, name=None, roles=None):
@@ -11,6 +15,24 @@ class JvURLPattern(URLPattern):
         return '<%s %s Roles:%s>' % (self.__class__.__name__, self.pattern.describe(), ",".join(self.roles) if self.roles else 'no-def')
 
 
+def grant_roles(request, roles):
+    if isinstance(roles, str):
+        roles = [roles]
+    if isinstance(roles, tuple):
+        roles = list(roles)
+    if not isinstance(roles, list):
+        raise Exception('roles must be string, tuple or list')
+    if request and hasattr(request, 'session'):
+        if request.session.exists(role_session_key):
+            s_roles = request.session[role_session_key]
+            s_roles = list(set(s_roles).union(set(roles)))
+            request.session.set(role_session_key, s_roles)
+        else:
+            request.session.set(role_session_key, roles)
+        logger = logging.getLogger("jhvar.django.logger")
+        logger.debug("Current session roles are %s" % ",".join(request.session[role_session_key]))
+
+
 def _jhvar_path(route, view, kwargs=None, name=None, Pattern=None, roles=None):
     if isinstance(view, (list, tuple)):
         # For include(...) processing.
